@@ -22,10 +22,14 @@
 #define INTER_NODEPS  (1 << 0)
 #define INTER_NOORDER (1 << 1)  /* only for install */
 
+typedef void (*rpmInstallCb)(gchar *name, gchar *group, void *user_data);
+
 int do_install(char *root, GList *pkgs, char *location,
-	       int installFlags, int interfaceFlags);
+	       int installFlags, int interfaceFlags,
+	       rpmInstallCb callback, void *user_data);
 int install_one(char *root, char *file, char *location,
-		int installFlags, int interfaceFlags);
+		int installFlags, int interfaceFlags,
+		rpmInstallCb callback, void *user_data);
 
 int do_uninstall(char *root, GList *indices, int uninstallFlags,
 		 int interfaceFlags);
