@@ -61,6 +61,9 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
 			    gint x, gint y, GtkSelectionData *selection_data,
 			    guint info, guint time, RpmPackageList *pl);
 
+static void mainwin_context_menu(RpmPackageList *pl, GdkEventButton *event,
+				 guint index);
+
 static guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
 
 #ifdef WITH_RPMFIND
@@ -107,6 +110,8 @@ GtkWidget *create_main(char *app_id, DBHandle *hdl) {
   gnome_app_set_contents(GNOME_APP(app), pl);
   gtk_widget_show(pl);
 
+  gtk_signal_connect(GTK_OBJECT(pl), "context_menu",
+		     GTK_SIGNAL_FUNC(mainwin_context_menu), NULL);
 
   status = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER);
   gnome_app_set_statusbar(GNOME_APP(app), status);
@@ -284,6 +289,9 @@ static void rpm_uninstall_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   db_handle_db_down(pl->hdl);
   do_uninstall(pl->hdl->root, pl->selection, uninstallFlags, interfaceFlags);
   db_handle_db_up(pl->hdl);
+  /* update the package list display */
+  rpm_package_list_clear_selection(pl);
+  rpm_package_list_update_pane(pl);
 }
 
 static void rpm_verify_pkgs(GtkWidget *mi, RpmPackageList *pl) {
@@ -296,6 +304,7 @@ static void verify_func(RpmQuery *info, gpointer data) {
 static void uninstall_func(RpmQuery *info, gpointer data) {
   int failed;
   char buf[512];
+  RpmPackageList *pl = RPM_PACKAGE_LIST(data);
 
   db_handle_db_down(info->hdl);
   failed = uninstall_one(info->hdl->root, info->index, uninstallFlags,
@@ -304,6 +313,17 @@ static void uninstall_func(RpmQuery *info, gpointer data) {
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
     message_box(buf);
+    return;
+  }
+  /* update the package list display */
+  rpm_package_list_update_pane(pl);
+  rpm_package_list_unselect(pl, info->index);
+
+  /* remove the package from the query window */
+  if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
+    GtkWidget *parent = GTK_WIDGET(info)->parent;
+
+    gtk_container_remove(GTK_CONTAINER(parent), GTK_WIDGET(info));
   }
 }
 
@@ -313,8 +333,8 @@ static void rpm_query_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   if (pl->selection == NULL) return;
   win = rpm_query_dialog_new(pl->hdl, pl->selection);
   rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(win), verify_func, NULL);
-  rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,NULL
-);
+  rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,
+				      pl);
   gtk_widget_show(win);
 }
 
@@ -339,7 +359,7 @@ static void rpm_find_query_cb(RpmFindDialog *dlg, GList *indices,
   win = rpm_query_dialog_new(pl->hdl, indices);
   rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(win), verify_func, NULL);
   rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,
-				      NULL);
+				      pl);
   gtk_widget_show(win);
 }
 
@@ -347,6 +367,8 @@ static void rpm_find_uninstall_cb(RpmFindDialog *dlg, GList *indices,
 			RpmPackageList *pl) {
   int failed;
   char buf[512];
+  GList *tmp;
+
   if (indices == NULL) return;
   db_handle_db_down(pl->hdl);
   failed = do_uninstall(pl->hdl->root, indices, uninstallFlags,interfaceFlags);
@@ -355,6 +377,10 @@ static void rpm_find_uninstall_cb(RpmFindDialog *dlg, GList *indices,
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
     message_box(buf);
   }
+  /* update the package list display */
+  rpm_package_list_update_pane(pl);
+  for (tmp = indices; tmp; tmp = tmp->next)
+    rpm_package_list_unselect(pl, GPOINTER_TO_UINT(tmp->data));
 }
 
 static void rpm_find_verify_cb(RpmFindDialog *dlg, GList *indices,
@@ -376,46 +402,56 @@ static void rpm_find(GtkWidget *mi, RpmPackageList *pl) {
   gtk_widget_show(find);
 }
 
-static void install_func(RpmQuery *info) {
+static void gnorpm_update_tree(gchar *pkgName, gchar *groupName, void *data) {
+  RpmPackageList *pl = RPM_PACKAGE_LIST(data);
+
+  rpm_package_list_add_group(pl, groupName);
+}
+
+static void install_func(RpmQuery *info, RpmPackageList *pl) {
   db_handle_db_down(info->hdl);
   if (install_one(info->hdl->root, info->fname, NULL, installFlags,
-		  interfaceFlags)) {
+		  interfaceFlags, gnorpm_update_tree, pl)) {
     gchar buf[512];
     g_snprintf(buf, 511, _("Couldn't install %s"), info->fname);
     message_box(buf);
   }
   db_handle_db_up(info->hdl);
+  rpm_package_list_update_pane(pl);
 }
-static void upgrade_func(RpmQuery *info) {
+static void upgrade_func(RpmQuery *info, RpmPackageList *pl) {
   db_handle_db_down(info->hdl);
   if (install_one(info->hdl->root, info->fname, NULL,
-		  installFlags | RPMINSTALL_UPGRADE, interfaceFlags)) {
+		  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+		  gnorpm_update_tree, pl)) {
     gchar buf[512];
     g_snprintf(buf, 511, _("Couldn't upgrade %s"), info->fname);
     message_box(buf);
   }
   db_handle_db_up(info->hdl);
+  rpm_package_list_update_pane(pl);
 }
 static void checksig_func(RpmQuery *info) {
   check_one_sig(info->fname, 0);
 }
 
 static void rpm_instpkgs_query(RpmInstallDialog *dlg, GList *files,
-			       DBHandle *hdl) {
+			       RpmPackageList *pl) {
   GtkWidget *query;
 
   if (files == NULL) return;
-  query = rpm_query_dialog_new_from_files(hdl, files);
+  query = rpm_query_dialog_new_from_files(pl->hdl, files);
   rpm_query_dialog_set_install_func(RPM_QUERY_DIALOG(query),
-				    (GtkRpmCallback)install_func, NULL);
+				    (GtkRpmCallback)install_func, pl);
   rpm_query_dialog_set_upgrade_func(RPM_QUERY_DIALOG(query),
-				    (GtkRpmCallback)upgrade_func, NULL);
+				    (GtkRpmCallback)upgrade_func, pl);
   rpm_query_dialog_set_checksig_func(RPM_QUERY_DIALOG(query),
 				      (GtkRpmCallback)checksig_func, NULL);
   gtk_widget_show(query);
 }
 
-static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files) {
+static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files,
+				RpmPackageList *pl) {
   gint res;
 
   if (files == NULL) {
@@ -423,13 +459,15 @@ static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files) {
     return;
   }
   db_handle_db_down(dlg->hdl);
-  res = do_install(dlg->hdl->root, files, NULL, installFlags, interfaceFlags);
+  res = do_install(dlg->hdl->root, files, NULL, installFlags, interfaceFlags,
+		   gnorpm_update_tree, pl);
   if (res) {
     gchar buf[512];
     g_snprintf(buf, 511, _("Install of %d packages failed."), res);
     message_box(buf);
   }
   db_handle_db_up(dlg->hdl);
+  rpm_package_list_update_pane(pl);
 }
 static void rpm_check_sigs_cb(RpmInstallDialog *dlg, GList *files) {
   if (files == NULL) {
@@ -444,9 +482,9 @@ static void rpm_install_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   win = rpm_install_dialog_new(pl->hdl);
   gtk_window_set_title(GTK_WINDOW(win), _("Install"));
   gtk_signal_connect(GTK_OBJECT(win), "query",
-		     GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl->hdl);
+		     GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl);
   gtk_signal_connect(GTK_OBJECT(win), "install",
-		     GTK_SIGNAL_FUNC(rpm_install_pkgs_cb), NULL);
+		     GTK_SIGNAL_FUNC(rpm_install_pkgs_cb), pl);
   gtk_signal_connect(GTK_OBJECT(win), "checksig",
 		     GTK_SIGNAL_FUNC(rpm_check_sigs_cb), NULL);
   gtk_widget_show(win);
@@ -466,7 +504,7 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
     gtk_signal_connect(GTK_OBJECT(win), "query",
 		       GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl->hdl);
     gtk_signal_connect(GTK_OBJECT(win), "install",
-		       GTK_SIGNAL_FUNC(rpm_install_pkgs_cb), NULL);
+		       GTK_SIGNAL_FUNC(rpm_install_pkgs_cb), pl);
     gtk_signal_connect(GTK_OBJECT(win), "checksig",
 		       GTK_SIGNAL_FUNC(rpm_check_sigs_cb), NULL);
     list = gnome_uri_list_extract_filenames(selection_data->data);
@@ -480,7 +518,8 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
 }
 
 
-static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files) {
+static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files,
+				RpmPackageList *pl) {
   gint res;
 
   if (files == NULL) {
@@ -489,13 +528,16 @@ static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files) {
   }
   db_handle_db_down(dlg->hdl);
   res = do_install(dlg->hdl->root, files, NULL,
-		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags);
+		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+		   gnorpm_update_tree, pl);
   if (res) {
     gchar buf[512];
     g_snprintf(buf, 511, _("Upgrade of %d packages failed."), res);
     message_box(buf);
   }
   db_handle_db_up(dlg->hdl);
+
+  rpm_package_list_update_pane(pl);
 }
 static void rpm_upgrade_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   GtkWidget *win;
@@ -504,7 +546,7 @@ static void rpm_upgrade_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   gtk_signal_connect(GTK_OBJECT(win), "query",
 		     GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl->hdl);
   gtk_signal_connect(GTK_OBJECT(win), "install",
-		     GTK_SIGNAL_FUNC(rpm_upgrade_pkgs_cb), NULL);
+		     GTK_SIGNAL_FUNC(rpm_upgrade_pkgs_cb), pl);
   gtk_widget_show(win);
   /* do_install( ... , installFlags | RPMINSTALL_UPGRADE ) */
 }
@@ -512,7 +554,7 @@ static void rpm_upgrade_pkgs(GtkWidget *mi, RpmPackageList *pl) {
 #ifdef WITH_RPMFIND
 static void rpm_show_web_find(GtkWidget *mi, RpmPackageList *pl) {
   GtkWidget *win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  GtkWidget *webfind = rpm_web_find_new(pl->hdl);
+  GtkWidget *webfind = rpm_web_find_new(pl);
 
   gtk_window_set_title(GTK_WINDOW(win), _("Rpmfind"));
   rpm_web_find_set_callbacks(RPM_WEB_FIND(webfind),
@@ -572,3 +614,64 @@ static void rpm_preferences_cb(GtkWidget *mi, RpmPackageList *pl) {
   }
 }
 
+static void context_query(GtkWidget *mi, gpointer index_p) {
+  guint index = GPOINTER_TO_UINT(index_p);
+  RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
+  GList item;
+  GtkWidget *win;
+
+  item.data = index_p;
+  item.next = item.prev = NULL;
+
+  win = rpm_query_dialog_new(pl->hdl, &item);
+  rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(win), verify_func, NULL);
+  rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,
+				      pl);
+  gtk_widget_show(win);
+}
+
+static void context_uninstall(GtkWidget *mi, gpointer index_p) {
+  guint index = GPOINTER_TO_UINT(index_p);
+  RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
+
+  db_handle_db_down(pl->hdl);
+  uninstall_one(pl->hdl->root, index, uninstallFlags, interfaceFlags);
+  db_handle_db_up(pl->hdl);
+}
+
+static void context_verify(GtkWidget *mi, gpointer index_p) {
+  guint index = GPOINTER_TO_UINT(index_p);
+  RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
+
+  verify_one(pl->hdl, index, 0);
+}
+
+static GnomeUIInfo context_menu[] = {
+    GNOMEUIINFO_ITEM_STOCK(N_("_Query"),
+			N_("Get information about the selected packages"),
+			context_query, RPM_STOCK_MENU_QUERY),
+    GNOMEUIINFO_ITEM_STOCK(N_("_Uninstall"),
+			N_("Uninstall the selected packages"),
+			context_uninstall, RPM_STOCK_MENU_UNINSTALL),
+    GNOMEUIINFO_ITEM_STOCK(N_("_Verify"),
+			N_("Verify the selected packages"),
+			context_verify, RPM_STOCK_MENU_VERIFY),
+    GNOMEUIINFO_END
+};
+
+static void mainwin_context_menu(RpmPackageList *pl, GdkEventButton *event,
+				 guint index) {
+  GtkWidget *menu;
+
+  menu = gtk_object_get_data(GTK_OBJECT(pl), "context-menu");
+  if (!menu) {
+    menu = gnome_popup_menu_new(context_menu);
+    gnome_app_install_menu_hints(gnorpm_app, context_menu);
+    gtk_object_set_data(GTK_OBJECT(context_menu[0].widget), "package-list",pl);
+    gtk_object_set_data(GTK_OBJECT(context_menu[1].widget), "package-list",pl);
+    gtk_object_set_data(GTK_OBJECT(context_menu[2].widget), "package-list",pl);
+    gtk_object_set_data(GTK_OBJECT(pl), "context-menu", menu);
+  }
+  gnome_popup_menu_do_popup(menu, NULL, NULL, event, GUINT_TO_POINTER(index));
+}
+
